/* Parameters.h header file defining constants, classes
 * 
 * Authors: Feng Qian (fqian@sable.mcgill.ca)
 *          Marc Berndl (berndl@sable.mcgill.ca)
 *          Ondrej Lhotak (olhotak@sable.mcgill.ca)
 */

#ifndef ParametersFile
#define ParametersFile
/* Parameters parses the command line inputs and stores the results in a useable format */

#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>

enum VerbageLevel {silent, normal, verbose, debug};
enum FileType {assignmentsOnly,assignmentsAndClassInformation};
enum AssignmentType {objectAllocation,directAssignment,loadField,fieldStore,Assignment,nonAssignment,
		     unrecognized,ClassType,AllocType,VarType,FieldType,Solution,Comments};

enum VarOrdering {DEFAULT,           // 0 five domains are interleaved, least significant bit first
		  ORDERED,           // 1 variables are ordered, least significant bit first
		  REVERSED,          // 2          
		  FD_V1_V2_V3_HP,
		  FD_V1xV2_V3_HP,
		  FD_V1xV2xV3_HP,
		  V1xV2xV3_FD_HP,
		  HP_V1xV2xV3_FD,
		  FDxV1xV2xV3xHP,
		  V1xV2xV3xFDxHP,    
		  HP_V1xV2xV3xFD,    
		  V1xV2xV3xFD_HP,    
		  V1xV2xFD_HP_V3,    
		  V1_V2xFDxHPxV3,
		  V1xV2xV3_FDxHP,
		  FD_V1xV2xV3xHP,
		  RANDOM,            
		  RANDVARS,
		  RANDFD,
		  RANDHP,
		  RANDALL,

		  // for Solover-noalias
		  FD_V1_V2_H1_H2,
		  FD_V1xV2_H1_H2,
		  FD_V2_V1_H1_H2,
		  FD_V2xV1_H1_H2,
		  FDxV1xV2xH1xH2,
		  FD_H1_V1xV2_H2,
		  FD_H1_V2xV1_H2,
		  H1_V1xV2_FD_H2,
		  H1_V2xV1_FD_H2,
		  FD_V1xV2xH1_H2,		  
};        


enum VarEncoding {ENUMERATION, CLSMETHVAR};

enum ControlCode {ITERATION, TRANSCLOSURE, REACHABILITY, PARTITIONED};

enum AliasStrategy {MERGING, SEPARATE, NOALIAS, FLOWONLY};

enum IncrementalOptimization {INCREMENTAL,NONINCREMENTAL};

class Parameters{
 public:
  static const char usageString[];
  static const bool defaultTypeFilter=false;
  static const int defaultNumNodes=1000000;
  static const int defaultCacheSize=100000;
  static const VerbageLevel defaultDetailLevel=normal;
  static const VarEncoding defaultVarEncoding=ENUMERATION;
  static const ControlCode defaultControlCode=ITERATION;
  static const AliasStrategy defaultAliasStrategy=SEPARATE;
  static const IncrementalOptimization defaultIncrementalization=INCREMENTAL;
  static const bool defaultReversedLocalOrdering = true;
  
  ControlCode ctrlcode;
  AliasStrategy aliasstrategy;
  IncrementalOptimization incremental;
  char * orderingName;

  char * inputFileName;
  FILE * inputFile;
  char * outputFileName;
  static ostream * outputFile;

  // solution for verification
  char * solutionFileName;
  FILE * solutionFile; 

  FILE * currentInputFile;
 
  int numNodes;
  int cacheSize;
  int numberOfProgramArguments;
  char ** programArguments;
  bool typeFilter;
  int varordering;
  int varencoding;

  bool reversedLocalOrdering;
  
  static VerbageLevel detailLevel;
  
  static bool thisShouldBePrinted(VerbageLevel importance);
  static void output(char * key, char * string, VerbageLevel importance);
  static void output(char * key, int value,VerbageLevel importance);
  
  FileType theFileType;
  
  Parameters(int count, char * strings[]);
  FileType determineFileTypeFromFile();
  bool verifyAndLoadFileStructure(int & locals,int & objects, int & fields);
  AssignmentType getValues(int &pc, int &pm, int &pv, int &qc, int &qm, int &qv, int &f);
  AssignmentType getType(char * line);	
  void prepareLoadInput();
  void prepareLoadSolution();
  void dumpParameters();
};
#endif
